library(paleotree)
library(geiger)
library(phytools)




timetree<-read.nexus(file.choose())  #######Read in tree
data.matrix<-read.csv(file.choose(),header=F,row.name=1,na.strings="?")####read in character matrix
data.matrix<-data.matrix[timetree$tip.label,] ####reorder matrix to order of tips in phylogeny


#####
## Pagel's Lambda
#####


lambda<-array(dim=ncol(data.matrix),data=NA)#####empty array to store lambda values

for(i in 1:length(lambda))
{
	character<-as.vector(data.matrix[,i])   ######convert character into vector
	names(character)<-timetree$tip.label  ########give vector taxon names
	drop<-tree$tip.label[which(is.na(character)==T)] ######identify taxa with "?" score  
	if(length(drop)<(length(timetree$tiplabel)/4) #####Check if too many taxa are being dropped
	{
		test.tree<-drop.tip(timetree,drop) #####drop taxa scored "?" from tree
		character<-character[test.tree$tip.label] ######drop from character list
		if(length(unique(character))>1)#### Check that there is more than one unique character state stil represented
		{
			model.fit<-fitDiscrete(test.tree, character, transform="lambda") ##### calculate pagel's lambda
			tooth.lambda[i]<-model.fit$opt$lambda######## store the lambda score
		}
	}
}






#####
## Within and Between character rate heterogeneity
#####

aic<-array(dim=ncol(characters),data=NA) ####empty array to store aic values
rates<-array(dim=ncol(characters),data=NA)######empty array to store rates


for(i in 1:length(aic))
{
	character<-as.vector(data.matrix[,i])######convert character into vector
	names(character)<-timetree$tip.label ######give vector taxon names 
	drop<-tree$tip.label[which(is.na(character)==T)] ######identify taxa with "?" score  
	if(length(drop)<(length(timetree$tiplabel)/4) #####Check if too many taxa are being dropped
	{
		test.tree<-drop.tip(timetree,drop) #####drop taxa scored "?" from tree
		character<-character[test.tree$tip.label] ######drop from character list
		if(length(unique(character))>1)#### Check that there is more than one unique character state stil represented
		{

			er.fit<-fitDiscrete(test.tree, character, model="ER") #####fit ER model to data
			ard.fit<-fitDiscrete(test.tree, character, model="ARD")#####fit ARD model to data
			aicc<-c(er.fit$opt$aicc,ard.fit$opt$aicc) #####extract AIC values
			tooth.aic[i]<-aicw(aicc)[1,3]#####calculate akaike weights

			if(tooth.aic[i]>0.5) #####check if character better fits ER model
			{
				rates[[i]]<-er.fit$opt[[1]]#####store rates
			}
			
			else 
			{
				sim<-make.simmap(test.tree, character, model="ARD", nsim=1000) ####simmmap models of character
				rates<-vector(length=length(sim))
				for(j in 1:length(sim))	
				{
					count<-countSimmap(sim[[j]], states=NULL, message=TRUE)$Tr ######count number of character changes
					rates[j]<-sum(count)/sum(test.tree$edge.length) ##### calculate weighted mean character rate
				}
		
				rates[[i]]<-mean(rates)#####store rate
			}

		}
	}
	print(i)
}

#####
### Character independance
#####





pagel<-vector(length=0) ##### vectore of length 0 to which results will be added


for(i in 1:ncol(data.matrix))
{
	for(j in i:ncol(data.matrix))
	{
		if(i!=j) #### make sure not comparing two of the same character
		{
			char.1<-data.matrix[,i]   ###### extract first character
			char.1<-char.1[which(is.na(char.1)==F)] #####get rid of  "?" scores
			char.2<-data.matrix[,j] #### extract second character
			char.2<-char.2[which(is.na(char.2)==F)] #####get rid of "?" scores
			overlap<-intersect(names(char.1),names(char.2)) #####get taxa with known states for both characters
			char.1<-char.1[overlap] ##### limit to overlapping taxa
			char.2<-char.2[overlap]	#### limit to overlapping taxa
			
			if(length(unique(char.1))==2 && length(unique(char.2))==2) #####make sure both characters are binary and variable
			{

				
				drop<-timetree$tip.label[which(timetree$tip.label%in%overlap==F)] #####identifytaxa that have been dropped 
				if(length(drop)<(length(timetree$tiplabel)/4)) ####make sure not too many taxa have been dropped
				{
					test.tree<-drop.tip(timetree,drop)####drop those taxa from tree
					
					pagel<-fitPagel(test.tree,char.1,char.2,model="ER")#####apply method of pagel 
					weights<-aicw(c(pagel$independent.AIC,pagel$dependent.AIC))######Calculate Akaike weights
					tooth.pagel<-c(tooth.pagel,weights[1,3])##### store weights of independnat model
				}
			}
		}
	}
}













